/*
Copyright (c) 2017-present NAVER Corp.
name: @egjs/vue-view360
license: MIT
author: NAVER Corp.
repository: https://github.com/naver/egjs-view360/tree/master/packages/vue-view360
version: 3.0.0
*/
import Vue from 'vue';
import { PANOVIEWER_OPTIONS, PanoViewer as PanoViewer$1, withPanoViewerMethods, PANOVIEWER_EVENTS, updatePanoViewer, DEFAULT_CANVAS_CLASS, SPINVIEWER_OPTIONS, SpinViewer as SpinViewer$1, withSpinViewerMethods, SPINVIEWER_EVENTS, DEFAULT_WRAPPER_CLASS, DEFAULT_IMAGE_CLASS } from '@egjs/view360';
export * from '@egjs/view360';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var getProps = function (thisVal) {
  return Object.keys(thisVal.$props).reduce(function (props, propName) {
    if (thisVal.$props[propName] != null) {
      props[propName] = thisVal.$props[propName];
    }

    return props;
  }, {});
};
var generateCanvasKey = function (oldKey) {
  var newKey;

  do {
    var array = new Uint32Array(1);
    crypto.getRandomValues(array);
    newKey = array[0];
  } while (newKey === oldKey);

  return newKey;
};

var PropDefintion$1 = Object.keys(PANOVIEWER_OPTIONS).reduce(function (props, optionName) {
  props[optionName] = null;
  return props;
}, {});
var PanoViewer = Vue.extend({
  props: __assign(__assign({}, PropDefintion$1), {
    tag: {
      type: String,
      required: false,
      default: "div"
    }
  }),
  created: function () {
    this._canvasKey = -1;
  },
  mounted: function () {
    var _this = this;

    var props = getProps(this);
    this._vanillaPanoViewer = new PanoViewer$1(this.$refs.container, props);
    withPanoViewerMethods(this, "_vanillaPanoViewer");
    this._prevProps = props;
    var panoViewer = this._vanillaPanoViewer;
    var events = Object.keys(PANOVIEWER_EVENTS).map(function (key) {
      return PANOVIEWER_EVENTS[key];
    });
    events.forEach(function (eventName) {
      panoViewer.on(eventName, function (e) {
        e.currentTarget = _this; // Make events from camelCase to kebab-case

        _this.$emit(eventName.replace(/([A-Z])/g, "-$1").toLowerCase(), e);
      });
    });
  },
  beforeDestroy: function () {
    var panoViewer = this._vanillaPanoViewer; // Turn off video if it has one

    var video = panoViewer === null || panoViewer === void 0 ? void 0 : panoViewer.getVideo();

    if (video) {
      video.pause();
    }

    panoViewer === null || panoViewer === void 0 ? void 0 : panoViewer.destroy();
  },
  updated: function () {
    this.$emit("updated");
  },
  watch: {
    $props: {
      handler: function () {
        var panoViewer = this._vanillaPanoViewer;
        if (!panoViewer) return;
        var nextProps = getProps(this);
        var prevProps = this._prevProps;

        if (nextProps.image != null && nextProps.image !== prevProps.image || nextProps.video != null && nextProps.video !== prevProps.video) {
          this._canvasKey = generateCanvasKey(this._canvasKey);
          this.$forceUpdate(); // Update after render

          this.$once("updated", function () {
            updatePanoViewer(panoViewer, nextProps, prevProps);
          });
        } else {
          // Update immediately
          updatePanoViewer(panoViewer, nextProps, prevProps);
        }

        this._prevProps = nextProps;
      },
      deep: true,
      immediate: true
    }
  },
  render: function (h) {
    var _a;

    var canvasClass = (_a = this.canvasClass) !== null && _a !== void 0 ? _a : DEFAULT_CANVAS_CLASS;
    return h(this.tag, {
      ref: "container"
    }, [h("canvas", {
      staticClass: canvasClass,
      key: this._canvasKey
    }), this.$slots.default]);
  }
});

var PropDefintion = Object.keys(SPINVIEWER_OPTIONS).reduce(function (props, optionName) {
  props[optionName] = null;
  return props;
}, {});
var SpinViewer = Vue.extend({
  props: __assign(__assign({}, PropDefintion), {
    tag: {
      type: String,
      required: false,
      default: "div"
    }
  }),
  mounted: function () {
    var _this = this;

    var props = getProps(this);
    this._vanillaSpinViewer = new SpinViewer$1(this.$refs.container, props);
    withSpinViewerMethods(this, "_vanillaSpinViewer");
    var spinViewer = this._vanillaSpinViewer;
    var events = Object.keys(SPINVIEWER_EVENTS).map(function (key) {
      return SPINVIEWER_EVENTS[key];
    });
    events.forEach(function (eventName) {
      spinViewer.on(eventName, function (e) {
        e.currentTarget = _this; // Make events from camelCase to kebab-case

        _this.$emit(eventName.replace(/([A-Z])/g, "-$1").toLowerCase(), e);
      });
    });
  },
  watch: {
    scale: function (newVal) {
      var _a;

      (_a = this._vanillaSpinViewer) === null || _a === void 0 ? void 0 : _a.setScale(newVal);
    }
  },
  render: function (h) {
    var _a, _b;

    var wrapperClass = (_a = this.wrapperClass) !== null && _a !== void 0 ? _a : DEFAULT_WRAPPER_CLASS;
    var imageClass = (_b = this.imageClass) !== null && _b !== void 0 ? _b : DEFAULT_IMAGE_CLASS;
    return h(this.tag, {
      ref: "container"
    }, [h("div", {
      staticClass: wrapperClass
    }, [h("img", {
      staticClass: imageClass
    })]), this.$slots.default]);
  }
});

var version = "3.0.0";

var install = function (Vue) {
  Vue.component("PanoViewer", PanoViewer);
  Vue.component("SpinViewer", SpinViewer);
};

var plugin = {
  PanoViewer: PanoViewer,
  SpinViewer: SpinViewer,
  install: install,
  version: version
};

export { PanoViewer, SpinViewer, plugin as default, install, version };
//# sourceMappingURL=view360.esm.js.map
